@@ -42,6 +42,7 @@
 #include <xf86Xinput.h>
 #include <exevents.h>
 #include <mipointer.h>
+#include <xorgVersion.h>
 
 #include "evdev.h"
 
@@ -252,8 +253,15 @@ EvdevReadInput(InputInfoPtr pInfo)
 		break;
 
             default:
-		if (ev.code > BTN_TASK && ev.code < KEY_OK)
+		if (ev.code > BTN_TASK && ev.code < KEY_OK) {
+		    /* Some fancy mice with a lot of buttons generate
+		     * button events between BTN_TASK and BTN_JOYSTICK */
+		    if (ev.code < BTN_JOYSTICK)
+			xf86PostButtonEvent(pInfo->dev, 0,
+			                    ev.code - BTN_LEFT + 5,
+			                    value, 0, 0);
 		    break;
+		}
 
                 PostKbdEvent(pInfo, &ev, value);
 		break;
@@ -298,7 +306,9 @@ EvdevReadInput(InputInfoPtr pInfo)
     }
 }
 
-#define TestBit(bit, array) (array[(bit) / 8] & (1 << ((bit) % 8)))
+#define LONG_BITS (sizeof(long) * 8)
+#define NBITS(x) (((x) + LONG_BITS - 1) / LONG_BITS)
+#define TestBit(bit, array) (array[(bit) / LONG_BITS]) & (1 << ((bit) % LONG_BITS))
 
 static void
 EvdevPtrCtrlProc(DeviceIntPtr device, PtrCtrl *ctrl)
@@ -834,12 +844,15 @@ EvdevProc(DeviceIntPtr device, int what)
         if (!pEvdev->kernel24 && ioctl(pInfo->fd, EVIOCGRAB, (void *)1))
             xf86Msg(X_WARNING, "%s: Grab failed (%s)\n", pInfo->name,
                     strerror(errno));
-        xf86AddEnabledDevice(pInfo);
-	if (pEvdev->flags & EVDEV_BUTTON_EVENTS)
-	    EvdevMBEmuPreInit(pInfo);
-	device->public.on = TRUE;
+        if (errno != ENODEV)
+        {
+            xf86AddEnabledDevice(pInfo);
+            if (pEvdev->flags & EVDEV_BUTTON_EVENTS)
+                EvdevMBEmuPreInit(pInfo);
+            device->public.on = TRUE;
+        }
 	break;
-	    
+
     case DEVICE_OFF:
         if (!pEvdev->kernel24 && ioctl(pInfo->fd, EVIOCGRAB, (void *)0))
             xf86Msg(X_WARNING, "%s: Release failed (%s)\n", pInfo->name,
@@ -884,9 +897,9 @@ EvdevConvert(InputInfoPtr pInfo, int first, int num, int v0, int v1, int v2,
 static int
 EvdevProbe(InputInfoPtr pInfo)
 {
-    char key_bitmask[(KEY_MAX + 7) / 8];
-    char rel_bitmask[(REL_MAX + 7) / 8];
-    char abs_bitmask[(ABS_MAX + 7) / 8];
+    long key_bitmask[NBITS(KEY_MAX)];
+    long rel_bitmask[NBITS(REL_MAX)];
+    long abs_bitmask[NBITS(ABS_MAX)];
     int i, has_axes, has_buttons, has_keys;
     EvdevPtr pEvdev = pInfo->private;
 
@@ -1091,8 +1104,8 @@ static XF86ModuleVersionInfo EvdevVersionRec =
     MODULEVENDORSTRING,
     MODINFOSTRING1,
     MODINFOSTRING2,
-    0, /* Missing from SDK: XORG_VERSION_CURRENT, */
-    1, 0, 0,
+    XORG_VERSION_CURRENT,
+    PACKAGE_VERSION_MAJOR, PACKAGE_VERSION_MINOR, PACKAGE_VERSION_PATCHLEVEL,
     ABI_CLASS_XINPUT,
     ABI_XINPUT_VERSION,
     MOD_CLASS_XINPUT,
